;;; Directory Local Variables
;;; For more information see (info "(emacs) Directory Variables")

((nil
  (copyright-names-regexp . "Free Software Foundation, Inc\\.")
  (sentence-end-double-space . t))
 (emacs-lisp-mode
  (indent-tabs-mode . nil)
  ;; extra config here: https://github.com/abo-abo/oremacs/blob/github/modes/ora-elisp-style-guide.el
  ;; (lisp-indent-function . common-lisp-indent-function)
  )
 (markdown-mode
  (fill-column . 70))
 (org-mode
  (fill-column . 70)))
