;; Generated package description from ivy.el  -*- no-byte-compile: t; lexical-binding:t -*-
(define-package "ivy" "0.15.1.0.20260213.120315" "Incremental Vertical completYon" '((emacs "24.5")) :commit "6c05de85416504095fd879c665697fdea5a24af1" :authors '(("Oleh Krehel" . "ohwoeowho@gmail.com")) :maintainer '("Basil L. Contovounesios" . "basil@contovou.net") :keywords '("matching") :url "https://github.com/abo-abo/swiper")
